/*
    qtparted - a frontend to libparted for manipulating disk partitions
    Copyright (C) 2002-2003 Vanni Brutto

    Vanni Brutto <zanac (-at-) libero dot it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qregexp.h>
#include "qp_fswrap.h"

#define NOTFOUND tr("command not found")

bool QP_FSWrap::fs_open(QString cmdline) {
    fp = popen(cmdline, "r");

    if (fp)
        return true;
    else
        return false;
}

char * QP_FSWrap::fs_getline() {
    bool rc = fgets(line, sizeof line, fp);

    if (rc) return line;
    else    return NULL;
}

int QP_FSWrap::fs_close() {
    return fclose(fp);
}

QP_FSWrap * QP_FSWrap::fswrap(QString name) {
    if (name.compare("ntfs") == 0) {
        QP_FSWrap *fswrap = new QP_FSNtfs();
        if (fswrap->wrap_enabled)
            return fswrap;
        else {
            delete fswrap;
            return NULL;
        }
    }
    else
        return NULL;
}


QP_FSNtfs::QP_FSNtfs() {
    wrap_min_size = true;
    wrap_resize = true;
    wrap_move = false;
    wrap_copy = false;
    wrap_create = false;
    wrap_enabled = false;

    /*---check if the wrapper is installed---*/
    QString cmdline = QString("which ntfsresize");
    fs_open(cmdline);

    char *cline;
    while ((cline = fs_getline()))
        wrap_enabled = true;
    fs_close();
}

/*---get how much data there is in the filesystem---*/
PedSector QP_FSNtfs::min_size(QString dev) {
    _message = QString::null;
    PedSector size = -1;
    QString cmdline = QString("ntfsresize -f -i %1")
                    .arg(dev);

    if (!fs_open(cmdline)) {
        _message = QString(NOTFOUND);
        return -1;
    }

    char *cline;
    while ((cline = fs_getline())) {
        QString line = QString(cline);

        QRegExp rx;
        rx = QRegExp("^ERROR: (.*)$");
        if (rx.search(line) == 0) {
            QString captured = rx.cap(1);
            _message = captured;
            return -1;
        }
          
        rx = QRegExp("^Space in use.*: (\\d*).*$");
        if (rx.search(line) == 0) {
            QString captured = rx.cap(1);

            bool rc;
            int mb_min_size = captured.toInt(&rc);
            size = mb_min_size * MEGABYTE_SECTORS;
        }
    }
    fs_close();

    return size;
}

bool QP_FSNtfs::resize(QString dev, PedSector newsize) {
    char szcmdline[200];
    QString cmdline;

    /*---init of the error message---*/
    _message = QString::null;

    /*---calculate size of the partition in bytes---*/
    PedSector size = (PedSector)((newsize-1)*512);

    /*---read-only test---*/
    sprintf(szcmdline, "ntfsresize -n -ff -s %lld %s", size, dev.latin1());
    cmdline = QString(szcmdline);

    if (!fs_open(cmdline)) {
        _message = QString(NOTFOUND);
        return false;
    }

    bool error = false;
    char *cline;
    while ((cline = fs_getline())) {
        QString line = QString(cline);

        QRegExp rx;

        if (!error) {
            rx = QRegExp("^ERROR: (.*)");
            if (rx.search(line) == 0) {
                QString captured = rx.cap(1);
                _message = QString(captured);
                error = true;
            }
        }

        if (!error) {
            rx = QRegExp("^The volume end is fragmented.*");
            if (rx.search(line) == 0) {
                _message = QString("The partition is fragmented.");
                error = true;
            }
        }

        rx = QRegExp("^Now You could resize at \\d* bytes or (\\d*) .*");
        if (rx.search(line) == 0) {
            QString captured = rx.cap(1);
            _message = QString("The partition is fragmented. Try to defragment it, or resize to %1MB")
                        .arg(captured);
            error = true;
        }
    }
    fs_close();

    if (error) return false;



    /*---ok, the readonly test seems ok... now we resize it!---*/
    sprintf(szcmdline, "ntfsresize -ff -s %lld %s", size, dev.latin1());
    cmdline = QString(szcmdline);

    if (!fs_open(cmdline)) {
        _message = QString(NOTFOUND);
        return false;
    }
    bool success = false;
    while ((cline = fs_getline())) {
        QString line = QString(cline);

        QRegExp rx;
        rx = QRegExp("^Successfully resized NTFS on device");
        if (rx.search(line) == 0)
            success = true;
        rx = QRegExp("^Nothing to do: NTFS volume size is already OK.");
        if (rx.search(line) == 0)
            success = true;
    }
    fs_close();

    if (success) return true;
    else {
        _message = QString("An error occured! :(");
        return false;
    }
}

bool QP_FSNtfs::move(QString, PedSector, PedSector) {
    return false;
}

bool QP_FSNtfs::mkpartfs(QString, QString) {
    return false;
}

QString QP_FSNtfs::message() {
    return _message;
}


